const Express = require('express')
const compression = require('compression')

const { IS_DEV, IS_PROD } = require('../config/env.config')
const appConfig = require('../utils/get-app-config')
const { publicPath } = require('../utils/index')

const proxyMiddleware = require('./middlewares/proxy')
const ssrMiddleware = require('./middlewares/ssr')

module.exports = function createApp(options, onStart = () => {}) {
  const app = Express()

  // 路由hash配置的时候，带路由的页面刷新会404，需要加入这个
  app.use(require('connect-history-api-fallback')())
  // 开启gzip
  app.use(compression())

  // 服务端静态资源路径
  if (!appConfig.deploy.isSSR) {
    app.use(Express.static(publicPath))
  }
  app.use(`/${publicPath}`, Express.static(publicPath))

  if (IS_DEV) {
    const webpackDevMiddleware = require('./middlewares/webpack.dev')
    webpackDevMiddleware(app)
  }

  if (IS_PROD && appConfig.deploy.isSSR) {
    app.use(ssrMiddleware)
  }

  // proxy配置
  app.use(proxyMiddleware)

  app.listen(appConfig.appPort, function () {
    console.log(`server is running in http://localhost:${appConfig.appPort}`)
  })

  // 服务器启动回调
  onStart()

  return app
}
