const fs = require('fs')
const LRU = require('lru-cache')
const { createBundleRenderer } = require('vue-server-renderer')
const { resolve, readFile, publicPath } = require('summerd-sdk/utils')

// const serve = (path, cache) => Express.static(resolve(path), {
//   maxAge: cache && isProd ? 1000 * 60 * 60 * 24 * 30 : 2000000
// })

const createRenderer = (bundle, options) => {
  return createBundleRenderer(bundle, Object.assign(options, {
    runInNewContext: false,
    cache: LRU({
      max: 1000,
      maxAge: 1000 * 60 * 15,
    }),
    basedir: resolve(publicPath),
  }))
}

const bundle = JSON.parse(readFile(fs, 'vue-ssr-server-bundle.json'))
const template = readFile(fs, 'index.html')
// const clientManifest = JSON.parse(readFile(fs, 'vue-ssr-client-manifest.json'))

// 服务端渲染

const renderer = createRenderer(bundle, {
  template,
})

const render = (req, res) => {
  const s = Date.now()
  res.setHeader('Content-Type', 'text/html')

  const context = {
    url: req.url,
  }

  renderer.renderToString(context, (err, html) => {
    if (err) {
      throw err
    }

    // const { title } = context.meta.inject()

    res.send(html)
  })
}

module.exports = function proxyMiddleware (req, res, next) {
  render(req, res)

  next()
}
